import traceback
from datetime import datetime

from cpl_core.configuration.configuration_model_abc import ConfigurationModelABC
from cpl_core.console import Console


class AuthenticationSettings(ConfigurationModelABC):
    def __init__(self):
        ConfigurationModelABC.__init__(self)

        self._secret_key = ""
        self._issuer = ""
        self._audience = ""
        self._token_expire_time = 0
        self._refresh_token_expire_time = 0

    @property
    def secret_key(self) -> str:
        return self._secret_key

    @property
    def issuer(self) -> str:
        return self._issuer

    @property
    def audience(self) -> str:
        return self._audience

    @property
    def token_expire_time(self) -> int:
        return self._token_expire_time

    @property
    def refresh_token_expire_time(self) -> int:
        return self._refresh_token_expire_time

    def from_dict(self, settings: dict):
        try:
            self._secret_key = settings["SecretKey"]
            self._issuer = settings["Issuer"]
            self._audience = settings["Audience"]
            self._token_expire_time = int(settings["TokenExpireTime"])
            self._refresh_token_expire_time = int(settings["RefreshTokenExpireTime"])
        except Exception as e:
            Console.error(f"[ ERROR ] [ {__name__} ]: Reading error in {type(self).__name__} settings")
            Console.error(f"[ EXCEPTION ] [ {__name__} ]: {e} -> {traceback.format_exc()}")
