from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.migration_abc import MigrationABC
from bot_data.db_context import DBContext


class ApiKeyMigration(MigrationABC):
    name = "1.0_ApiKeyMigration"

    def __init__(self, logger: DatabaseLogger, db: DBContext):
        MigrationABC.__init__(self)
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def upgrade(self):
        self._logger.debug(__name__, "Running upgrade")

        self._cursor.execute(
            str(
                f"""
            CREATE TABLE IF NOT EXISTS `ApiKeys` (
                `Id` BIGINT NOT NULL AUTO_INCREMENT,
                `Identifier` VARCHAR(255) NOT NULL,
                `Key` VARCHAR(255) NOT NULL,
                `CreatorId` BIGINT NULL,
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                PRIMARY KEY(`Id`),
                FOREIGN KEY (`CreatorId`) REFERENCES `Users`(`UserId`),
                CONSTRAINT UC_Identifier_Key UNIQUE (`Identifier`,`Key`),
                CONSTRAINT UC_Key UNIQUE (`Key`)
            );
            """
            )
        )

    def downgrade(self):
        self._cursor.execute("DROP TABLE `ApiKeys`;")
