from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.migration_abc import MigrationABC
from bot_data.db_context import DBContext


class UserJoinedGameServerMigration(MigrationABC):
    name = "1.0_UserJoinedGameServerMigration"

    def __init__(self, logger: DatabaseLogger, db: DBContext):
        MigrationABC.__init__(self)
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def upgrade(self):
        self._logger.debug(__name__, "Running upgrade")

        self._cursor.execute(
            str(
                f"""
            CREATE TABLE IF NOT EXISTS `UserJoinedGameServer` (
                `Id` BIGINT NOT NULL AUTO_INCREMENT,
                `UserId` BIGINT NOT NULL,
                `GameServer` VARCHAR(255) NOT NULL,
                `JoinedOn` DATETIME(6) NOT NULL,
                `LeavedOn` DATETIME(6),
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                FOREIGN KEY (`UserId`) REFERENCES Users(`UserId`),
                PRIMARY KEY(`Id`)
            );
            """
            )
        )

        self._cursor.execute(
            str(
                f"""
            ALTER TABLE Users ADD MinecraftId VARCHAR(255) NULL AFTER XP;
            """
            )
        )

    def downgrade(self):
        self._cursor.execute("DROP TABLE `UserJoinedGameServer`;")
        self._cursor.execute(
            str(
                f"""
            ALTER TABLE Users DROP COLUMN MinecraftId;
            """
            )
        )
