import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.logging import LoggerABC
from cpl_discord.events import OnVoiceStateUpdateABC
from cpl_translation import TranslatePipe

from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.helper.event_checks import EventChecks
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from modules.base.abc.base_helper_abc import BaseHelperABC
from modules.base.configuration.base_server_settings import BaseServerSettings
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class BaseOnVoiceStateUpdateEventHelpChannel(OnVoiceStateUpdateABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: LoggerABC,
        base_helper: BaseHelperABC,
        servers: ServerRepositoryABC,
        permissions: PermissionServiceABC,
        message_service: MessageServiceABC,
        t: TranslatePipe,
    ):
        OnVoiceStateUpdateABC.__init__(self)
        self._config = config
        self._logger = logger
        self._base_helper = base_helper
        self._servers = servers
        self._permissions = permissions
        self._message_service = message_service
        self._t = t

        self._logger.info(__name__, f"Module {type(self)} loaded")

    @EventChecks.check_is_ready()
    async def on_voice_state_update(
        self,
        member: discord.Member,
        before: discord.VoiceState,
        after: discord.VoiceState,
    ):
        self._logger.debug(__name__, f"Module {type(self)} started")
        server = self._servers.get_server_by_discord_id(member.guild.id)
        settings: BaseServerSettings = self._base_helper.get_config(server.discord_id)
        if after.channel is None or after.channel.id != settings.help_voice_channel_id:
            return

        mods = [
            *self._permissions.get_admins(member.guild.id),
            *self._permissions.get_moderators(member.guild.id),
        ]
        for a in mods:
            await self._message_service.send_dm_message(
                self._t.transform("modules.base.member_joined_help_voice_channel").format(member.mention),
                a,
            )

            self._logger.debug(__name__, f"Module {type(self)} stopped")
