import {Component} from '@angular/core';
import {AuthService} from "../../../../../../services/auth/auth.service";
import {SpinnerService} from "../../../../../../services/spinner/spinner.service";
import {ToastService} from "../../../../../../services/toast/toast.service";
import {ConfirmationDialogService} from "../../../../../../services/confirmation-dialog/confirmation-dialog.service";
import {FormBuilder, FormControl, FormGroup} from "@angular/forms";
import {TranslateService} from "@ngx-translate/core";
import {DataService} from "../../../../../../services/data/data.service";
import {SidebarService} from "../../../../../../services/sidebar/sidebar.service";
import {ActivatedRoute} from "@angular/router";
import {Page} from "../../../../../../models/graphql/filter/page.model";
import {Sort, SortDirection} from "../../../../../../models/graphql/filter/sort.model";
import {Level, LevelFilter} from "../../../../../../models/data/level.model";
import {LevelListQuery} from "../../../../../../models/graphql/query.model";
import {Queries} from "../../../../../../models/graphql/queries.model";
import {debounceTime} from "rxjs/operators";
import {LazyLoadEvent} from "primeng/api";

@Component({
  selector: 'app-levels',
  templateUrl: './levels.component.html',
  styleUrls: ['./levels.component.scss']
})
export class LevelsComponent {

  levels!: Level[];
  loading = true;

  isEditingNew: boolean = false;

  filterForm!: FormGroup<{
    id: FormControl<number | null>,
    name: FormControl<string | null>,
    color: FormControl<string | null>,
    min_xp: FormControl<number | null>,
    permissions: FormControl<number | null>,
  }>;

  filter: LevelFilter = {};
  page: Page = {
    pageSize: undefined,
    pageIndex: undefined
  };
  sort: Sort = {
    sortColumn: undefined,
    sortDirection: undefined
  };

  totalRecords!: number;

  public constructor(
    private authService: AuthService,
    private spinner: SpinnerService,
    private toastService: ToastService,
    private confirmDialog: ConfirmationDialogService,
    private fb: FormBuilder,
    private translate: TranslateService,
    private data: DataService,
    private sidebar: SidebarService,
    private route: ActivatedRoute) {
  }

  public ngOnInit(): void {
    this.data.getServerFromRoute(this.route);

    this.setFilterForm();
    this.loadNextPage();
  }

  public loadNextPage() {
    this.loading = true;
    this.data.query<LevelListQuery>(Queries.levelQuery, {
        filter: this.filter, page: this.page, sort: this.sort
      }
    ).subscribe(data => {
      this.totalRecords = data.levelCount;
      this.levels = data.levels;
      this.spinner.hideSpinner();
      this.loading = false;
    });
  }

  public setFilterForm() {
    this.filterForm = this.fb.group({
      id: new FormControl<number | null>(null),
      name: new FormControl<string | null>(null),
      color: new FormControl<string | null>(null),
      min_xp: new FormControl<number | null>(null),
      permissions: new FormControl<number | null>(null),
    });

    this.filterForm.valueChanges.pipe(
      debounceTime(600)
    ).subscribe(changes => {
      if (changes.id) {
        this.filter.id = changes.id;
      } else {
        this.filter.id = undefined;
      }

      if (changes.name) {
        this.filter.name = changes.name;
      } else {
        this.filter.name = undefined;
      }

      if (this.page.pageSize)
        this.page.pageSize = 10;

      if (this.page.pageIndex)
        this.page.pageIndex = 0;

      this.loadNextPage();
    });
  }

  public nextPage(event: LazyLoadEvent) {
    this.page.pageSize = event.rows ?? 0;
    if (event.first != null && event.rows != null)
      this.page.pageIndex = event.first / event.rows;
    this.sort.sortColumn = event.sortField ?? undefined;
    this.sort.sortDirection = event.sortOrder === 1 ? SortDirection.ASC : event.sortOrder === -1 ? SortDirection.DESC : SortDirection.ASC;

    this.loadNextPage();
  }

}
